@@ -98,7 +98,7 @@ int convert_format(buffer *in, sample_format *in_fmt, buffer *out, sample_format
 
 		int error;
 		// FIXME: The quality probably ought to be configurable.
-		src = src_new(SRC_SINC_BEST_QUALITY, channels, &error);
+		src = src_new(SRC_SINC_MEDIUM_QUALITY, channels, &error);
 		if (src == NULL)
 			g_error("error creating src: %d", error);
 
