@@ -26,6 +26,8 @@ enum endianness {
 	END_NATIVE,
 };
 
+#define BITS_FLOAT (-1)
+
 typedef struct sample_format {
 	int bits;
 	int rate;
@@ -34,9 +36,9 @@ typedef struct sample_format {
 } sample_format;
 
 int bytes_per_sample(int bits);
-void print_format(sample_format *fmt);
 int convert_format(buffer *in, sample_format *in_fmt, buffer *out, sample_format *out_fmt, int processing, const char **error);
 void copy_format(sample_format *out, const sample_format *in);
 int format_same(const sample_format *a, const sample_format *b);
+void append_format_str(GString *s, const sample_format *fmt);
 
 #endif
