@@ -1,6 +1,6 @@
 /*
  * Potamus: an audio player
- * Copyright (C) 2004, 2005, 2006, 2007 Adam Sampson <ats@offog.org>
+ * Copyright (C) 2004, 2005, 2006, 2007, 2009 Adam Sampson <ats@offog.org>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -107,7 +107,7 @@ static int input_avcodec_get_audio(input *p, buffer *buf) {
 	// Keep reading until we get some audio.
 	int out_size = 0, in_total = 0;
 	do {
-		AVPacket packet;
+		AVPacket packet, orig;
 
 		// Read until we get an audio frame.
 		while (1) {
@@ -122,26 +122,33 @@ static int input_avcodec_get_audio(input *p, buffer *buf) {
 			av_free_packet(&packet);
 		}
 
+		orig = packet;
+
 		// Decode the frame.
-		uint8_t *in_data = packet.data;
-		int in_size = packet.size;
-		while (in_size > 0) {
+		while (packet.size > 0) {
 			int chunk = AVCODEC_MAX_AUDIO_FRAME_SIZE;
 			unsigned char *out = buffer_reserve(buf, chunk);
 			if (out == NULL)
 				g_error("out of memory");
 
+#ifdef HAVE_AVCODEC_DECODE_AUDIO3
+			int in_used = avcodec_decode_audio3(a->codec_ctx,
+							    (int16_t *)out,
+							    &chunk, &packet);
+#else
 			int in_used = avcodec_decode_audio2(a->codec_ctx,
 							    (int16_t *)out,
 							    &chunk,
-							    in_data, in_size);
+							    packet.data,
+							    packet.size);
+#endif
 			if (in_used < 0) {
 				// Decoding error -- skip this frame.
 				break;
 			}
 
-			in_data += in_used;
-			in_size -= in_used;
+			packet.data += in_used;
+			packet.size -= in_used;
 			in_total += in_used;
 			buf->used += chunk;
 			out_size += chunk;
@@ -149,7 +156,7 @@ static int input_avcodec_get_audio(input *p, buffer *buf) {
 
 		a->pos = packet.pts;
 
-		av_free_packet(&packet);
+		av_free_packet(&orig);
 	} while (out_size == 0);
 
 	p->fmt.bits = 16;
