@@ -1,6 +1,6 @@
 /*
  * Potamus: an audio player
- * Copyright (C) 2004, 2005, 2006, 2007 Adam Sampson <ats@offog.org>
+ * Copyright (C) 2004, 2005, 2006, 2007, 2010 Adam Sampson <ats@offog.org>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -173,6 +173,11 @@ void update_status() {
 		g_string_append(s, "Paused");
 		break;
 	case STATE_PLAYING:
+		if (pstatus.error != NULL) {
+			g_string_append(s, pstatus.error);
+			break;
+		}
+
 		if (pstatus.bitrate >= 0.0) {
 			history[history_pos] = pstatus.bitrate;
 			history_pos = (history_pos + 1) % 10;
@@ -408,10 +413,19 @@ gboolean handle_seek_ungrab(GtkWidget *w, GdkEventButton *e, gpointer d) {
 	return FALSE;
 }
 
+static gboolean handle_error_continue(gpointer data) {
+	status_set_error(&pstatus, NULL, FALSE);
+	update_ui();
+
+	move_direction(1);
+
+	return FALSE;
+}
+
 void handle_player_finished() {
 	if (pstatus.error == NULL) {
 		move_direction(1);
-	} else {
+	} else if (pstatus.error_stop) {
 		stop_playing();
 		update_ui();
 
@@ -424,11 +438,15 @@ void handle_player_finished() {
 		gtk_dialog_run(GTK_DIALOG(dialog));
 		gtk_widget_destroy(dialog);
 
-		status_set_error(&pstatus, NULL);
+		status_set_error(&pstatus, NULL, FALSE);
+	} else {
+		update_ui();
+
+		g_timeout_add(500, handle_error_continue, NULL);
 	}
 }
 
-gboolean handle_timeout(gpointer data) {
+gboolean handle_periodic_update(gpointer data) {
 	set_can_seek(pstatus.seekable);
 	set_length(MAX(pstatus.len, pstatus.pos));
 	if (!(seek_in_progress || seek_grabbed)) {
@@ -625,11 +643,12 @@ int main(int argc, char **argv) {
 	g_signal_connect(G_OBJECT(list), "row-activated",
 	                 G_CALLBACK(handle_row_activated), NULL);
 
-	g_timeout_add (100, handle_timeout, NULL);
+	g_timeout_add(100, handle_periodic_update, NULL);
 
 	for (int i = 1; i < argc; i++)
 		add_recursively(argv[i]);
 
+	move_direction(1);
 	update_ui();
 
 	gtk_main();
