@@ -1,6 +1,6 @@
 /*
  * Potamus: an audio player
- * Copyright (C) 2004, 2005, 2006, 2007 Adam Sampson <ats@offog.org>
+ * Copyright (C) 2004, 2005, 2006, 2007, 2009 Adam Sampson <ats@offog.org>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -45,12 +45,10 @@ static int oj_play(struct output *o, buffer *buf, sample_format *fmt) {
 	output_jack *a = (output_jack *) o->data;
 
 	while (buf->used > 0) {
-		size_t n = jack_ringbuffer_write_space(a->ringbuf);
-		if (n > buf->used)
-			n = buf->used;
-
 		// Move data into the ringbuf (as is).
-		jack_ringbuffer_write(a->ringbuf, (const char *) buf->data, n);
+		size_t n = jack_ringbuffer_write(a->ringbuf,
+		                                 (const char *) buf->data,
+		                                 buf->used);
 		buffer_remove(buf, n);
 
 		if (n == 0) {
@@ -136,6 +134,8 @@ output *output_new_jack(sample_format **force_fmt) {
 	output_jack *a = malloc(sizeof *a);
 	if (a == NULL)
 		g_error("out of memory");
+	a->client = NULL;
+	a->ringbuf = NULL;
 	o->data = a;
 
 	o->play = oj_play;
