@@ -1,6 +1,6 @@
 /*
  * Potamus: an audio player
- * Copyright (C) 2004, 2005, 2006, 2007 Adam Sampson <ats@offog.org>
+ * Copyright (C) 2004, 2005, 2006, 2007, 2009 Adam Sampson <ats@offog.org>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -71,10 +71,11 @@ static void status_reset(player_status *status) {
 	status->converting = 0;
 }
 
-void status_set_error(player_status *status, const char *error) {
+void status_set_error(player_status *status, const char *error, gboolean stop) {
 	if (status->error != NULL)
 		free(status->error);
 	status->error = (error == NULL) ? NULL : strdup(error);
+	status->error_stop = stop;
 }
 
 // --- Player thread code ---
@@ -119,6 +120,15 @@ static input *get_input_for_file(const char *fn) {
 static gpointer player_thread_func(gpointer data) {
 	player_status *status = (player_status *) data;
 
+	sample_format *force_fmt = NULL;
+	output *out = get_output(&force_fmt);
+	if (out == NULL) {
+		status_set_error(status, "No output method available", TRUE);
+		if (write(player_pipe[1], "f", 1) != 1)
+			g_error("error writing to pipe");
+		return NULL;
+	}
+
 	// NULL means not currently playing.
 	input *p = NULL;
 	gboolean finished = FALSE;
@@ -131,13 +141,6 @@ static gpointer player_thread_func(gpointer data) {
 	if (play_buf == NULL)
 		g_error("out of memory");
 
-	sample_format *force_fmt = NULL;
-	output *out = get_output(&force_fmt);
-	if (out == NULL) {
-		status_set_error(status, "No output method available");
-		goto finished;
-	}
-
 	while (1) {
 		player_message *msg;
 
@@ -154,11 +157,11 @@ static gpointer player_thread_func(gpointer data) {
 					g_error("PLAY while playing");
 				p = get_input_for_file(msg->u.s);
 				if (p == NULL) {
-					status_set_error(status, "Unknown file type");
+					status_set_error(status, "Unknown file type", FALSE);
 					goto finished;
 				}
 				if (p->open(p, msg->u.s) < 0) {
-					status_set_error(status, "Cannot open file");
+					status_set_error(status, "Cannot open file", FALSE);
 					goto finished;
 				}
 				free(msg->u.s);
@@ -192,7 +195,7 @@ static gpointer player_thread_func(gpointer data) {
 				if (p == NULL)
 					g_error("SEEK while not playing");
 				if (p->set_pos(p, msg->u.d) < 0) {
-					status_set_error(status, "Cannot seek in file");
+					status_set_error(status, "Cannot seek in file", TRUE);
 					goto finished;
 				}
 				break;
@@ -208,7 +211,7 @@ static gpointer player_thread_func(gpointer data) {
 			// Once the file's finished *and* the buffers are empty, we're done.
 			goto finished;
 		} else if (n < 0) {
-			status_set_error(status, "Cannot read audio from file");
+			status_set_error(status, "Cannot read audio from file", TRUE);
 			goto finished;
 		}
 
@@ -228,7 +231,7 @@ static gpointer player_thread_func(gpointer data) {
 		if (status->processing != PROCESS_NONE || !format_same(&p->fmt, output_fmt)) {
 			const char *error;
 			if (convert_format(read_buf, &p->fmt, play_buf, output_fmt, status->processing, &error) < 0) {
-				status_set_error(status, error);
+				status_set_error(status, error, TRUE);
 				goto finished;
 			}
 			output_buf = play_buf;
@@ -240,7 +243,7 @@ static gpointer player_thread_func(gpointer data) {
 		copy_format(&status->out_fmt, output_fmt);
 
 		if (out->play(out, output_buf, output_fmt) < 0) {
-			status_set_error(status, "Cannot output audio");
+			status_set_error(status, "Cannot output audio", TRUE);
 			goto finished;
 		}
 
