@@ -30,8 +30,9 @@ typedef struct {
 	int converting;
 	int processing;
 	char *error;
+	gboolean error_stop;
 } player_status;
-void status_set_error(player_status *status, const char *error);
+void status_set_error(player_status *status, const char *error, gboolean stop);
 
 void player_init(int *argc, char ***argv, player_status *status);
 gboolean player_start(const gchar *path);
